import { IIterator } from '@phosphor/algorithm';
import { ISignal } from '@phosphor/signaling';
import { IAjaxSettings } from '../utils';
import { Kernel } from './kernel';
/**
 * An implementation of a kernel manager.
 */
export declare class KernelManager implements Kernel.IManager {
    /**
     * Construct a new kernel manager.
     *
     * @param options - The default options for kernel.
     */
    constructor(options?: Kernel.IOptions);
    /**
     * A signal emitted when the specs change.
     */
    readonly specsChanged: ISignal<this, Kernel.ISpecModels>;
    /**
     * A signal emitted when the running kernels change.
     */
    readonly runningChanged: ISignal<this, Kernel.IModel[]>;
    /**
     * Test whether the terminal manager is disposed.
     */
    readonly isDisposed: boolean;
    /**
     * Dispose of the resources used by the manager.
     */
    dispose(): void;
    /**
     * Get the base url of the manager.
     */
    readonly baseUrl: string;
    /**
     * Get the ws url of the manager.
     */
    readonly wsUrl: string;
    /**
     * The default ajax settings for the manager.
     */
    /**
     * Set the default ajax settings for the manager.
     */
    ajaxSettings: IAjaxSettings;
    /**
     * Get the most recently fetched kernel specs.
     */
    readonly specs: Kernel.ISpecModels | null;
    /**
     * Test whether the manager is ready.
     */
    readonly isReady: boolean;
    /**
     * A promise that fulfills when the manager is ready.
     */
    readonly ready: Promise<void>;
    /**
     * Create an iterator over the most recent running kernels.
     *
     * @returns A new iterator over the running kernels.
     */
    running(): IIterator<Kernel.IModel>;
    /**
     * Force a refresh of the specs from the server.
     *
     * @returns A promise that resolves when the specs are fetched.
     *
     * #### Notes
     * This is intended to be called only in response to a user action,
     * since the manager maintains its internal state.
     */
    refreshSpecs(): Promise<void>;
    /**
     * Force a refresh of the running kernels.
     *
     * @returns A promise that with the list of running sessions.
     *
     * #### Notes
     * This is not typically meant to be called by the user, since the
     * manager maintains its own internal state.
     */
    refreshRunning(): Promise<void>;
    /**
     * Start a new kernel.  See also [[startNewKernel]].
     *
     * @param options - Overrides for the default options.
     */
    startNew(options?: Kernel.IOptions): Promise<Kernel.IKernel>;
    /**
     * Find a kernel by id.
     *
     * @param options - Overrides for the default options.
     */
    findById(id: string, options?: Kernel.IOptions): Promise<Kernel.IModel>;
    /**
     * Connect to a running kernel.  See also [[connectToKernel]].
     *
     * @param options - Overrides for the default options.
     */
    connectTo(id: string, options?: Kernel.IOptions): Promise<Kernel.IKernel>;
    /**
     * Shut down a kernel by id.
     *
     * @param options - Overrides for the default options.
     *
     * #### Notes
     * This will emit [[runningChanged]] if the running kernels list
     * changes.
     */
    shutdown(id: string, options?: Kernel.IOptions): Promise<void>;
    /**
     * Handle a kernel terminating.
     */
    private _onTerminated(id);
    /**
     * Handle a kernel starting.
     */
    private _onStarted(kernel);
    /**
     * Refresh the specs.
     */
    private _refreshSpecs();
    /**
     * Refresh the running sessions.
     */
    private _refreshRunning();
    /**
     * Get optionally overidden options.
     */
    private _getOptions(options?);
    private _baseUrl;
    private _wsUrl;
    private _token;
    private _ajaxSettings;
    private _running;
    private _specs;
    private _isDisposed;
    private _runningTimer;
    private _specsTimer;
    private _readyPromise;
    private _isReady;
    private _specsChanged;
    private _runningChanged;
}
