import { JSONObject } from '@phosphor/coreutils';
import { IDisposable } from '@phosphor/disposable';
import { ISignal } from '@phosphor/signaling';
import { Contents, ContentsManager } from './contents';
import { Kernel } from './kernel';
import { Session, SessionManager } from './session';
import { TerminalSession, TerminalManager } from './terminal';
import { IAjaxSettings } from './utils';
/**
 * A Jupyter services manager.
 */
export declare class ServiceManager implements ServiceManager.IManager {
    /**
     * Construct a new services provider.
     */
    constructor(options?: ServiceManager.IOptions);
    /**
     * A signal emitted when the kernel specs change.
     */
    readonly specsChanged: ISignal<this, Kernel.ISpecModels>;
    /**
     * Test whether the terminal manager is disposed.
     */
    readonly isDisposed: boolean;
    /**
     * Dispose of the resources used by the manager.
     */
    dispose(): void;
    /**
     * The kernel spec models.
     */
    readonly specs: Kernel.ISpecModels | null;
    /**
     * Get the base url of the server.
     */
    readonly baseUrl: string;
    /**
     * Get the session manager instance.
     */
    readonly sessions: SessionManager;
    /**
     * Get the contents manager instance.
     */
    readonly contents: ContentsManager;
    /**
     * Get the terminal manager instance.
     */
    readonly terminals: TerminalManager;
    /**
     * Test whether the manager is ready.
     */
    readonly isReady: boolean;
    /**
     * A promise that fulfills when the manager is ready.
     */
    readonly ready: Promise<void>;
    private _sessionManager;
    private _contentsManager;
    private _terminalManager;
    private _isDisposed;
    private _readyPromise;
    private _specsChanged;
}
/**
 * The namespace for `ServiceManager` statics.
 */
export declare namespace ServiceManager {
    /**
     * A service manager interface.
     */
    interface IManager extends IDisposable {
        /**
         * A signal emitted when the kernel specs change.
         */
        specsChanged: ISignal<IManager, Kernel.ISpecModels>;
        /**
         * The kernel spec models.
         */
        readonly specs: Kernel.ISpecModels | null;
        /**
         * The base url of the manager.
         */
        readonly baseUrl: string;
        /**
         * The session manager for the manager.
         */
        readonly sessions: Session.IManager;
        /**
         * The contents manager for the manager.
         */
        readonly contents: Contents.IManager;
        /**
         * The terminals manager for the manager.
         */
        readonly terminals: TerminalSession.IManager;
        /**
         * Test whether the manager is ready.
         */
        readonly isReady: boolean;
        /**
         * A promise that fulfills when the manager is initially ready.
         */
        readonly ready: Promise<void>;
    }
    /**
     * The options used to create a service manager.
     */
    interface IOptions extends JSONObject {
        /**
         * The base url of the server.
         */
        baseUrl?: string;
        /**
         * The base ws url of the server.
         */
        wsUrl?: string;
        /**
         * The authentication token for the API.
         */
        token?: string;
        /**
         * The ajax settings for the manager.
         */
        ajaxSettings?: IAjaxSettings;
    }
}
