import { IIterator } from '@phosphor/algorithm';
import { ISignal } from '@phosphor/signaling';
import { Kernel } from '../kernel';
import { IAjaxSettings } from '../utils';
import { Session } from './session';
/**
 * An implementation of a session manager.
 */
export declare class SessionManager implements Session.IManager {
    /**
     * Construct a new session manager.
     *
     * @param options - The default options for each session.
     */
    constructor(options?: Session.IOptions);
    /**
     * A signal emitted when the kernel specs change.
     */
    readonly specsChanged: ISignal<this, Kernel.ISpecModels>;
    /**
     * A signal emitted when the running sessions change.
     */
    readonly runningChanged: ISignal<this, Session.IModel[]>;
    /**
     * Test whether the terminal manager is disposed.
     */
    readonly isDisposed: boolean;
    /**
     * Dispose of the resources used by the manager.
     */
    dispose(): void;
    /**
     * The base url of the manager.
     */
    readonly baseUrl: string;
    /**
     * The base ws url of the manager.
     */
    readonly wsUrl: string;
    /**
     * The default ajax settings for the manager.
     */
    /**
     * Set the default ajax settings for the manager.
     */
    ajaxSettings: IAjaxSettings;
    /**
     * Get the most recently fetched kernel specs.
     */
    readonly specs: Kernel.ISpecModels | null;
    /**
     * Test whether the manager is ready.
     */
    readonly isReady: boolean;
    /**
     * A promise that fulfills when the manager is ready.
     */
    readonly ready: Promise<void>;
    /**
     * Create an iterator over the most recent running sessions.
     *
     * @returns A new iterator over the running sessions.
     */
    running(): IIterator<Session.IModel>;
    /**
     * Force a refresh of the specs from the server.
     *
     * @returns A promise that resolves when the specs are fetched.
     *
     * #### Notes
     * This is intended to be called only in response to a user action,
     * since the manager maintains its internal state.
     */
    refreshSpecs(): Promise<void>;
    /**
     * Force a refresh of the running sessions.
     *
     * @returns A promise that with the list of running sessions.
     *
     * #### Notes
     * This is not typically meant to be called by the user, since the
     * manager maintains its own internal state.
     */
    refreshRunning(): Promise<void>;
    /**
     * Start a new session.  See also [[startNewSession]].
     *
     * @param options - Overrides for the default options, must include a
     *   `'path'`.
     */
    startNew(options: Session.IOptions): Promise<Session.ISession>;
    /**
     * Find a session by id.
     */
    findById(id: string, options?: Session.IOptions): Promise<Session.IModel>;
    /**
     * Find a session by path.
     */
    findByPath(path: string, options?: Session.IOptions): Promise<Session.IModel>;
    connectTo(id: string, options?: Session.IOptions): Promise<Session.ISession>;
    /**
     * Shut down a session by id.
     */
    shutdown(id: string, options?: Session.IOptions): Promise<void>;
    /**
     * Get optionally overidden options.
     */
    private _getOptions(options?);
    /**
     * Handle a session terminating.
     */
    private _onTerminated(id);
    /**
     * Handle a session starting.
     */
    private _onStarted(session);
    /**
     * Handle a change to a session.
     */
    private _onChanged(model);
    /**
     * Refresh the specs.
     */
    private _refreshSpecs();
    /**
     * Refresh the running sessions.
     */
    private _refreshRunning();
    private _baseUrl;
    private _wsUrl;
    private _ajaxSettings;
    private _isDisposed;
    private _running;
    private _specs;
    private _runningTimer;
    private _specsTimer;
    private _readyPromise;
    private _specsChanged;
    private _runningChanged;
}
