import { IIterator } from '@phosphor/algorithm';
import { ISignal } from '@phosphor/signaling';
import { IAjaxSettings } from '../utils';
import * as utils from '../utils';
import { TerminalSession } from './terminal';
/**
 * A terminal session manager.
 */
export declare class TerminalManager implements TerminalSession.IManager {
    /**
     * Construct a new terminal manager.
     */
    constructor(options?: TerminalManager.IOptions);
    /**
     * A signal emitted when the running terminals change.
     */
    readonly runningChanged: ISignal<this, TerminalSession.IModel[]>;
    /**
     * Test whether the terminal manager is disposed.
     */
    readonly isDisposed: boolean;
    /**
     * The base url of the manager.
     */
    readonly baseUrl: string;
    /**
     * The base ws url of the manager.
     */
    readonly wsUrl: string;
    /**
     * The default ajax settings for the manager.
     */
    /**
     * Set the default ajax settings for the manager.
     */
    ajaxSettings: IAjaxSettings;
    /**
     * Test whether the manger is ready.
     */
    readonly isReady: boolean;
    /**
     * Dispose of the resources used by the manager.
     */
    dispose(): void;
    /**
     * A promise that fulfills when the manager is ready.
     */
    readonly ready: Promise<void>;
    /**
     * Whether the terminal service is available.
     */
    isAvailable(): boolean;
    /**
     * Create an iterator over the most recent running terminals.
     *
     * @returns A new iterator over the running terminals.
     */
    running(): IIterator<TerminalSession.IModel>;
    /**
     * Create a new terminal session.
     *
     * @param ajaxSettings - The ajaxSettings to use, overrides manager
     *   settings.
     *
     * @returns A promise that resolves with the terminal instance.
     *
     * #### Notes
     * The baseUrl and wsUrl of the options will be forced
     * to the ones used by the manager. The ajaxSettings of the manager
     * will be used unless overridden.
     */
    startNew(options?: TerminalSession.IOptions): Promise<TerminalSession.ISession>;
    connectTo(name: string, options?: IAjaxSettings): Promise<TerminalSession.ISession>;
    /**
     * Shut down a terminal session by name.
     */
    shutdown(name: string): Promise<void>;
    /**
     * Force a refresh of the running sessions.
     *
     * @returns A promise that with the list of running sessions.
     *
     * #### Notes
     * This is not typically meant to be called by the user, since the
     * manager maintains its own internal state.
     */
    refreshRunning(): Promise<void>;
    /**
     * Handle a session terminating.
     */
    private _onTerminated(name);
    /**
     * Handle a session starting.
     */
    private _onStarted(session);
    /**
     * Refresh the running sessions.
     */
    private _refreshRunning();
    /**
     * Get a set of options to pass.
     */
    private _getOptions(options?);
    private _baseUrl;
    private _wsUrl;
    private _ajaxSettings;
    private _running;
    private _isDisposed;
    private _isReady;
    private _refreshTimer;
    private _readyPromise;
    private _runningChanged;
}
/**
 * The namespace for TerminalManager statics.
 */
export declare namespace TerminalManager {
    /**
     * The options used to initialize a terminal manager.
     */
    interface IOptions {
        /**
         * The base url.
         */
        baseUrl?: string;
        /**
         * The base websocket url.
         */
        wsUrl?: string;
        /**
         * The authentication token for the API.
         */
        token?: string;
        /**
         * The Ajax settings used for server requests.
         */
        ajaxSettings?: utils.IAjaxSettings;
    }
}
